#!/usr/bin/env bash
# Centova Cast Manual Installation Script
# Copyright 2007-2008, Centova Technologies Inc.
# ============================================================================
#
# This script is only used if you are installing manually (i.e., not from
# RPMs).
#
#
# OPTIONS
#
# If you want to install Centova Cast under an alternate username, change the
# following line.  Note that this is used ONLY when installing as root.
DEFUSERNAME=centovacast

# If you've already created the user account above, set the following to 0.
# Again, this is used ONLY when installing as root.
CREATEUSER=1

# If you have your PHP CLI binary installed in an unusual location, set the
# pathname here.
#
# NOTE: This does not need to be modified if your PHP CLI binary is found in
# your search path, or in /bin/php, /usr/bin/php, or /usr/local/bin/php.
PHPBINARY=php

# Optional arguments to pass to the PHP CLI.
PHPARGS=

# ============================================================================
# Installer code follows.
# You should not need to edit anything below this line.

# shoutcast download settings
SCLIC=http://www.shoutcast.com/license
SCSERV=http://download.nullsoft.com/shoutcast/tools/sc_serv_1.9.8_Linux.tar.gz
SCPATH=/home/centovacast/shoutcast/

function show_help {
	cat <<EOF
Installation Help
===============================================================================
This installation script has many options allowing advanced users to fully
customize the installation.  These options are described at:
  http://www.centova.com/pages/faqs/display/installation_options_86

If you are looking for installation instructions for Centova Cast, please see:
  http://www.centova.com/pages/castdocs/

If you are having problems installing Centova Cast, and/or you are using a web
hosting control panel (CPanel, Plesk, DirectAdmin, etc.) on this same server,
please refer to the following article:
  http://www.centova.com/pages/faqs/display/general_installation_89

EOF
	exit 1
}

# defaults
IGNOREDEPS=0
AUTOINSTALL=0
CPINSTALL=0
CCROOT="/home/centovacast"
WEBPATH=""
CCSERVER=""
SERVERPATH=""
CCSOURCE=""
SOURCEPATH=""
CPDOMAIN=""
CCSUPHP=""
FTPAUTO=""
DOWNLOADSHOUT=""
DOWNLOADICES=""
IGNOREPANEL=1
CANBUILDICES=0
CCINSTALLURL=""
SHOUTAGREE=""

while [ "$1" != "${1##[-+]}" ]; do
	case $1 in
		-h)
			show_help
			;;			
		--help)
			show_help
			;;
		--corepath=?*)
			CCROOT=${1#--corepath=}
			shift
			;;
		--path=?*)
			WEBPATH=${1#--path=}
			shift
			;;
		--servertype=?*)
			CCSERVER=${1#--servertype=}
			shift
			;;
		--serverpath=?*)
			SERVERPATH=${1#--serverpath=}
			shift
			;;
		--sourcetype=?*)
			CCSOURCE=${1#--sourcetype=}
			shift
			;;
		--shoutcast=?*)
			CCSERVER=ShoutCast
			SERVERPATH=${1#--shoutcast=}
			shift
			;;
		--icecast=?*)
			CCSERVER=IceCast
			SERVERPATH=${1#--icecast=}
			shift
			;;
		--sourcepath=?*)
			SOURCEPATH=${1#--sourcepath=}
			shift
			;;
		--domain=?*)
			CPDOMAIN=${1#--domain=}
			shift
			;;
		--user=?*)
			DEFUSERNAME=${1#--user=}
			shift
			;;
		--php=?*)
			PHPBINARY=${1#--php=}
			shift
			;;
		--cpinstall=?*)
			CPINSTALL=1
			AUTOINSTALL=1
			shift
			;;
		--ignore=?*)
			shift
			;;
		--url=?*)
			CCINSTALLURL=${1#--url=}
			shift
			;;
		--suphp=?*)
			CCSUPHP=${1#--suphp=}
			shift
			;;
		--shout)
			DOWNLOADSHOUT=y
			CCSERVER=ShoutCast
			shift
			;;
		--ices)
			DOWNLOADICES=y
			shift
			;;
		--ftp)
			FTPAUTO=y
			shift
			;;
		--canbuildices)
			CANBUILDICES=1
			shift
			;;
		--fullauto)
			DOWNLOADSHOUT=y
			SHOUTAGREE=y
			CCSERVER=ShoutCast
			DOWNLOADICES=y
			FTPAUTO=y
			AUTOINSTALL=1
			shift
			;;
		--auto)
			DOWNLOADSHOUT=y
			CCSERVER=ShoutCast
			DOWNLOADICES=y
			FTPAUTO=y
			AUTOINSTALL=1
			shift
			;;
		--ignorepanel)
			IGNOREPANEL=1
			shift
			;;
		--suphpuser=?*)
			shift
			;;
		--ignore=?*)
			shift
			;;
        *)
        	echo "Unrecognized option: $1"
        	exit 1
			;;
	esac
done

if [ ! -z "$SERVERPATH" ]; then
	if [ "${SERVERPATH:0:1}" != "/" ]; then
		MYPATH=`pwd`
		SERVERPATH="${MYPATH}/${SERVERPATH}"
	fi
	[ -d $SERVERPATH ] && echo "$CCSERVER pathname $SERVERPATH is a directory, not an executable file" && exit 1
	[ ! -x $SERVERPATH ] && echo "$CCSERVER path $SERVERPATH does not exist or is not executable" && exit 1

fi

function verify_exists {
	[ $IGNOREDEPS -gt 0 ] && return
	TESTFILE=`which $1`
	if [ -z "$TESTFILE" ]; then
		echo "Error: Installer requires a working copy of $2 in your path."
		[ ! -z "$3" ] && echo $3
		exit 1
	fi
}

# find out who we're running as
MYUID=`id -u`
[ $MYUID -eq 0 ] && ISROOT=1 || ISROOT=0

# are we FreeBSD? (todo: add support for other BSDs)
KERNOSTYPE=`/sbin/sysctl -n kern.ostype 2>&1`
[ "$KERNOSTYPE" == "FreeBSD" ] && ISBSD=1 || ISBSD=0

if [ $ISBSD -gt 0 ]; then
	# setup BSD utility paths
	USERADD="pw useradd"
	GROUPADD="pw groupadd"
	COPYCMD="cp -pPR"
	STATCMD="stat -f '%Su:%Sg'"
	SEDRCMD="sed -E"
	
	# verify BSD-specific dependecies
	[ $ISROOT -gt 0 ] && verify_exists pw "the pw utility"
	
else
	# setup Linux utility paths
	USERADD=useradd
	GROUPADD=groupadd
	COPYCMD="cp -a"
	STATCMD="stat -c %U:%G"
	SEDRCMD="sed -r"
	
	# verify Linux-specific dependencies
	[ $ISROOT -gt 0 ] && verify_exists $USERADD "the $USERADD utility"
	[ $ISROOT -gt 0 ] && verify_exists $GROUPADD "the $GROUPADD utility"
fi

# verify common dependencies
verify_exists gcc "gcc (the GNU C compiler)"
verify_exists sed "the sed utility"
verify_exists grep "the grep utility"

# make sure php exists
if [ "$PHPBINARY" == "php" ]; then
	verify_exists php "the PHP CLI binary" "Please specify the path to php using the --php parameter, eg:

$0 $* --php=/path/to/php"


else
	if [ ! -x "$PHPBINARY" ]; then
		echo "$PHPBINARY was not found.  Please specify the path to
php using the --php parameter, eg:

$0 $* --php=/path/to/php
"
		exit 1
	fi
fi

# detect popular web hosting control panels
[ -d /usr/local/cpanel ] && HASCPANEL=1 || HASCPANEL=0
[ -d /usr/local/psa ] && HASPLESK=1 || HASPLESK=0
[ -d /usr/local/direectadmin ] && HASDA=1 || HASDA=0
if [ $HASPLESK -gt 0 ]; then
	[ -d /home/httpd/vhosts ] && PLESKROOT=/home/httpd
	[ -d /var/www/vhosts ] && PLESKROOT=/var/www
fi

SUPPORTEDPANEL=0
[[ $HASCPANEL -gt 0 || $HASPLESK -gt 0 || $HASDA -gt -0 ]] && SUPPORTEDPANEL=1
[ $IGNOREPANEL -gt 0 ] && SUPPORTEDPANEL=0
CPNAME=""
[ $HASCPANEL -gt 0 ] && CPNAME="CPanel"
[ $HASPLESK -gt 0 ] && CPNAME="Plesk"
[ $HASDA -gt 0 ] && CPNAME="DirectAdmin"


# find php
if [ "$PHPBINARY" == "php" ]; then
	[ -f /usr/bin/which ] && WHICHPHP=`which php` || WHICHPHP=""
	[[ "$WHICHPHP" == "/bin/php" || "$WHICHPHP" == "/usr/bin/php" || "$WHICHPHP" == "/usr/local/bin/php" ]] && WHICHPHP=""

	PHPFOUND=0
	PHPMULTIPATHS=""
	PHPMULTIBINS=""
	PHPPATHS="/usr/local/bin/php /usr/bin/php /bin/php $WHICHPHP /usr/local/bin/php5 /usr/bin/php5 /bin/php5 /usr/local/bin/php4 /usr/bin/php4 /bin/php4"


	for f in $PHPPATHS; do
		if [ -f $f ]; then
			PHPFOUND=$(($PHPFOUND + 1))
			if [ $PHPFOUND -eq 1 ]; then
				PHPBINARY=$f
			fi
			PHPIDENT=`$f -v | grep '^PHP'`
			ISCLI=`echo "$PHPIDENT" | grep -c '(cli)'`;
			RECOMMEND=""
			[ $ISCLI -gt 0 ] && RECOMMEND="*** RECOMMENDED"
			PHPMULTIBINS="$PHPMULTIBINS $f"
			PHPMULTIPATHS="$PHPMULTIPATHS
$f - $PHPIDENT $RECOMMEND"
		fi
	done
	
	if [ $PHPFOUND -eq 0 ]; then
		echo "
The PHP CLI binary was not found on your server.  Please specify the path to
php using the --php parameter, eg:

$0 $* --php=$PHPBINARY
"
		exit 1
	fi
	
	
	# on some systems, you'll find both a CLI and CGI binary installed in the path;
	# if those are the only two PHP binaries installed, and both are of matching
	# verrsions, then we automatically select the CLI binary and ignore the other
	if [ $PHPFOUND -eq 2 ]; then
		LASTVER="-"
		ALLMATCH=1
		CLIBIN=""
		for f in $PHPMULTIBINS; do
			PHPVER=`$f -v | grep '^PHP' | awk '{print $2}'`
			PHPCLI=`$f -v | grep '^PHP' | grep -c '(cli)'`
			[ $PHPCLI -gt 0 ] && CLIBIN=$f
			
			if [ "$LASTVER" == "-" ]; then
				LASTVER="$PHPVER"
			else
				[ "$LASTVER" != "$PHPVER" ] && ALLMATCH=0
			fi
		done
		
		if [[ $ALLMATCH -gt 0 && ! -z "$CLIBIN" ]]; then
			PHPMULTI=0
			PHPFOUND=1
			PHPBINARY=$CLIBIN
		fi
	fi
	
	if [ $PHPFOUND -gt 1 ]; then
		echo "
More than one PHP commandline interpreter (CLI) was found on this server in the
following locations:
$PHPMULTIPATHS

Please re-run this installation script with the --php parameter to specify
the path to the correct PHP CLI which matches your web server's version of PHP.
For example:

$0 $* --php=$PHPBINARY

This will ensure the correct operation of Centova Cast.
"
		exit 1
	fi
	
fi

PHPNOARGS="$PHPBINARY"
PHPBINARY="$PHPBINARY $PHPARGS"

# If the admin tried to pass off the PHP CGI binary as the PHP CLI binary, we
# need to account for that, because if Apache is configured to use PHP via CGI,
# and Centova Cast's web interface then tries to launch a CLI (CGI) process,
# the CLI process is repeatedly launched until system resources are exhausted
# due to an apparent bug in PHP
PHPCGI=`$PHPNOARGS -v | grep '^PHP' | grep -c '(cgi)'`

# determine if we're able to autodownload ices-cc and shoutcast
WGETPATH=wget
CURLPATH=curl
HASWGET=`$WGETPATH -V 2>/dev/null | grep -ci 'gnu wget'`
HASCURL=`$CURLPATH -V 2>/dev/null | grep -ciE '^curl [0-9]+\.'`

if [ $HASWGET -gt 0 ]; then
	HTTPGET="$WGETPATH -O"
elif [ $HASCURL -gt 0 ]; then
	HTTPGET="$CURLPATH -f -o"
else
	HTTPGET=""
fi


# do a little self-exploration and discovery
MYUSER=`whoami`
SELFPATH=`dirname "$0"`
cd $SELFPATH

MYCWD=`pwd`
[ "${SELFPATH:0:1}" != "/" ] && SELFPATH="${MYCWD}/${SELFPATH}"
SELFPATH=`echo "${SELFPATH}" | sed 's/\/\+/\//g' | sed 's/\/\.//g'`



# find out if this is a "known" OS
HASDEBIAN=0
HASREDHAT=0

DIST=""
if [ -f /etc/debian_version ]; then
	HASDEBIAN=1
	DIST=deb
elif [ -f /etc/fedora-release ] ; then
	HASREDHAT=1
	REV=`cat /etc/fedora-release | sed s/.*release\ // | sed s/\ .*//`
	if [[ $REV -lt 5 || $REV -gt 8 ]]; then
		HASREDHAT=0
	fi
	DIST=fc${REV}
elif [ -f /etc/redhat-release ] ; then
	ISRHPROPER=`cat /etc/redhat-release | grep -ci 'red hat'`
	ISENT=`cat /etc/redhat-release | grep -ci enterprise`

	if [[ $ISRHPROPER -eq 0 || $ISENT -gt 0 ]]; then
		REV=`cat /etc/redhat-release | sed s/.*release\ // | sed s/\ .*// | sed 's/\..*//'`
		if [[ $REV -lt 4 || $REV -gt 5 ]]; then
			HASREDHAT=0
		else
			HASREDHAT=1
		fi
		DIST=el${REV}
	fi
fi

# no 64bit support in our ices-cc RPMs
IS64BIT=`uname -m | grep -ci x86_64`
[ -d /usr/lib64 ] && IS64BIT=1
[ $IS64BIT -gt 0 ] && HASREDHAT=0 && HASDEBIAN=0


# make sure PHP is sane
#
# Note: Centova Technologies regularly receives tickets from clients who have
# tried to "outsmart" the installer by commenting out the test below.
#
# In the majority of cases, commenting out the next line will yield a successful
# installation, but will result in a variety of problems when you attempt to use
# Centova Cast.  Be advised that Centova Technologies reserves the right to
# refuse to provide technicial support for installations wherein this test has
# been disabled or bypassed.
#
# If the test below fails, your PHP CLI is broken, and the correct solution
# is to fix your PHP CLI.  Please refer to the following knowledge base article
# for further information:
# http://www.centova.com/pages/faqs/display/testing_your_php_cli_binary_44
#

# before changing/removing this, read the note above
echo '<?php echo "test"; ?>' > _cc_install_test.php
PHPTEST=`$PHPBINARY -q _cc_install_test.php 2>&1`
rm -f _cc_install_test.php

# before changing/removing this, read the note above
if [ "x$PHPTEST" != "xtest" ]; then
	echo "Your PHP CLI binary does not appear to be working correctly.  Please"
	echo "correct this problem and then restart this installation script."
	exit 1
fi

# perform some initial tests to make sure MySQL support is installed,
# PHP is configured correctly, etc.
CHK=`$PHPBINARY -q $SELFPATH/system/installutil.php --check 2>&1`
if [ "x$CHK" != "xOK" ]; then
	echo "One or more errors were detected with your system configuration:"
	echo ""
	echo "$CHK"
	echo ""
	echo "Please correct the above problems and then restart this installation"
	echo "script."
	exit 1
fi

# greetings, friend!
echo ""
echo "===== CENTOVA CAST INSTALLER =================================================="

if [ $ISROOT -gt 0 ]; then
	INSTALLUSER=$DEFUSERNAME	
else
	echo ""
	echo ""
	echo "You are running Centova under the $MYUSER account."
	echo ""
	echo "Centova Cast must be installed as root.  Please login to the root account"
	echo "and try again."
	echo ""
	exit 1
fi


if [ $AUTOINSTALL -gt 0 ]; then
	# don't prompt for automated installation during automated installation :)
	SUPPORTEDPANEL=0
else
	echo "Welcome to the Centova Cast installer."
	echo ""
fi


echo "To begin, the installer needs to know the full URL at which you will be using"
echo "Centova Cast.  This is where your users will login to manage their streams."
echo ""

INSTALLURL=""
while [ -z "$INSTALLURL" ]; do
	if [ ! -z "$CCINSTALLURL" ]; then
		echo "Using: $CCINSTALLURL"
		INSTALLURL="$CCINSTALLURL"
		CCINSTALLURL=""
	else

		cat <<EOF
For example, you might enter: http://www.example.com/cast/
Or if using a subdomain, you might enter: http://streams.example.com/

Please ensure that you have setup this web site BEFORE proceeding with this
installation.

EOF

		echo -n "Enter the URL for Centova Cast: "

		read INSTALLURL
		
		ISOK=`echo "$INSTALLURL" | grep -cE '^http\s?://'`
		if [ $ISOK -eq 0 ]; then
			echo ""
			echo "Sorry, that does not look like a valid URL.  Please see the examples below and"
			echo "try again."
			echo ""
			INSTALLURL=""
		fi

		
	fi
done
echo ""
echo ""

INSTALLDOMAIN=`echo "$INSTALLURL" | $SEDRCMD 's|^https?://||' | sed 's|/.*||'`
INSTALLPATH=`echo "$INSTALLURL" | $SEDRCMD 's|^https?://||' | $SEDRCMD 's|^[^/]+||'`


if [ $SUPPORTEDPANEL -gt 0 ]; then
	CPCONFIG=`$PHPBINARY -q $SELFPATH/system/installutil.php --controlpanel ${HASPLESK}-${HASCPANEL}-${HASDA} "$INSTALLURL" 2>&1`
	CPOK=`echo "$CPCONFIG" | grep -c '^OK '`;
	
	if [ $CPOK -gt 0 ]; then
		SERVARG="--php=$PHPNOARGS"
		[ ! -z "$CCSOURCE" ] && SERVARG="--sourcetype=$CCSOURCE --sourcepath=$SOURCEPATH $SERVARG"
		[ ! -z "$CCSERVER" ] && SERVARG="--servertype=$CCSERVER --serverpath=$SERVERPATH $SERVARG"
		ALLARGS="--cpinstall=$CPCONFIG --domain=$DOMAINNAME $SERVARG"
		
		NEEDSUPHP=`echo "$CPCONFIG" | grep 'suphp=1'`
		if [ ! -z "$NEEDSUPHP" ]; then
			SUPHPUSER=`echo "$CPCONFIG" | sed 's/.*--suphpuser=//g' | sed 's/ .*//g'`
			USERCHECK=`cat /etc/passwd | grep "^$SUPHPUSER"`
			if [ -z "$USERCHECK" ]; then
				echo "Your control panel indicates that this web site runs under the $SUPHPUSER user"
				echo "account, but this account does not exist.  Aborting."
				exit 1
			fi
		fi

		$0 $ALLARGS
		exit $?
	fi
fi

echo "===== WEB ROOT PATH ==========================================================="
echo ""
echo "Please enter the path to the directory which corresponds to this URL:"
echo "$INSTALLURL"
echo ""

WEBROOT=""
if [ ! -z "$WEBPATH" ]; then
	WEBROOT=$WEBPATH
	echo "Installing in: $WEBROOT"
fi

CREATEDWEBROOT=0
while [ -z "$WEBROOT" ]; do
	
	echo "For example, if your web root for $INSTALLDOMAIN is /var/www/html"
	echo "then you might enter: /var/www/html${INSTALLPATH}"
	echo ""
	echo -n "Enter the path for your Centova Cast web interface [$WEBROOT]:"
	read NEWWEBROOT
	[ ! -z "$NEWWEBROOT" ] && WEBROOT="$NEWWEBROOT"
	if [ -z "$WEBROOT" ]; then
		echo "You must specify a path for the web interface."
		echo ""
	else
		CREATEDWEBROOT=0
		if [ ! -d $WEBROOT ]; then
			echo -n "Directory $WEBROOT does not exist.  Create it? [Yes] "
			read -n 1 CANCREATE
			echo ""
			if [[ -z "$CANCREATE" || "x$CANCREATE" == "xy" || "x$CANCREATE" == "xY" ]]; then
				CREATEDWEBROOT=1
				mkdir -p $WEBROOT
				if [ -d $WEBROOT ]; then
					# set it up to be owned by the same owner as the parent directory
					WEBROOTOWNER=`$STATCMD $WEBROOT/..`
					chown $WEBROOTOWNER $WEBROOT
				else
					echo "Error: Could not create directory: $WEBROOT"
					WEBROOT=""
				fi
			else
				WEBROOT=""
			fi
		fi
	fi

done



if [ -z "$CCSUPHP" ]; then

	echo ""
	echo "===== SUPHP DETECTION ========================================================="
	echo "Attempting to detect suPHP from $INSTALLURL ..."

	[ ! -d "$WEBROOT" ] && echo "$WEBROOT does not exist, aborting" && exit 1

	echo "<?php echo \`id\`; ?>" > $WEBROOT/_ccast_suphp_test.php
	WEBROOTOWNER=`$STATCMD $WEBROOT`
	chown $WEBROOTOWNER $WEBROOT/_ccast_suphp_test.php
	
	GETINFO=`$HTTPGET $SELFPATH/idoutput $INSTALLURL/_ccast_suphp_test.php 2>&1`
	EXITCODE=$?
	
	rm -f $WEBROOT/_ccast_suphp_test.php
	
	if [ $EXITCODE -gt 0 ]; then
		echo ""
		echo "Could not determine whether suPHP is in use.  You will need to manually"
		echo "determine whether suPHP is enabled on your server."
	else
		CCSUPHP=`cat $SELFPATH/idoutput | grep -vcE 'uid=[0-9]+\((nobody|httpd|www-data|www|apache)\)'`
		if [ $CCSUPHP -gt 0 ]; then
			echo "suPHP detected"
		else
			echo "suPHP not detected"
		fi
		echo ""
	fi
	
	rm -f $SELFPATH/idoutput

fi

# find out of suphp is being used (as that requires a non-runascc installation)
USESUPHP=""
while [ -z "$USESUPHP" ]; do
	if [ ! -z "$CCSUPHP" ]; then
		USESUPHP=$CCSUPHP
		CCSUPHP=""
	else
		echo ""
		echo "Is your web server configured to use suPHP (or phpsuexec)?"
		echo "(If you are unsure, consult with your systems administrator.)"
		echo ""
		echo -n "Press Y if suPHP or phpsuexec are enabled, otherwise press N: "
		read -n 1 USESUPHP
		echo ""
	fi
	case "$USESUPHP" in
		[yY1])	USESUPHP=1
				;;
		[nN0])	USESUPHP=0
				;;
		*)		USESUPHP=""
				echo ""
				echo ""
				echo "Please choose Y or N."
				echo ""
				;;
	esac
done
echo ""


if [ $CREATEUSER -gt 0 ]; then
	echo "Creating account $INSTALLUSER ..."
	$GROUPADD $INSTALLUSER
	$USERADD -s /bin/false -g $INSTALLUSER $INSTALLUSER
	USERADDRESULT=$?
	if [ $USERADDRESULT -eq 1 ]; then
		echo "The account $INSTALLUSER already exists."
		if [ $AUTOINSTALL -eq 0 ]; then
			echo "Press ENTER if you want to use the existing account, or Ctrl+C to exit."
			read
		fi
	fi
fi	
INSTALLGROUP=`groups $INSTALLUSER | awk '{print $1}'`


echo "Copying files ..."
mkdir -p $CCROOT
if [ ! -d $CCROOT ]; then
	echo "Error creating Centova Cast path: $CCROOT"
	exit 1
fi
chmod 0755 $CCROOT


mkdir -p $CCROOT/vhosts
$COPYCMD $SELFPATH/vhosts $CCROOT/
[ $ISROOT -gt 0 ] && chown -R $INSTALLUSER:$INSTALLGROUP $CCROOT/vhosts
chmod -R 755 $CCROOT/vhosts
mkdir -p $CCROOT/system
$COPYCMD $SELFPATH/system $CCROOT/
[ $ISROOT -gt 0 ] && chown -R $INSTALLUSER:$INSTALLGROUP $CCROOT/system
chmod -R 700 $CCROOT/system/skel/*
chmod 755 $CCROOT/system/skel/var
chmod -R 755 $CCROOT/system/skel/var/spool
mkdir -p $CCROOT/ioncube
$COPYCMD $SELFPATH/ioncube $CCROOT/
[ $ISROOT -gt 0 ] && chown -R $INSTALLUSER:$INSTALLGROUP $CCROOT/ioncube
mkdir -p $CCROOT/scripts
$COPYCMD $SELFPATH/scripts $CCROOT/
[ $ISROOT -gt 0 ] && chown -R $INSTALLUSER:$INSTALLGROUP $CCROOT/scripts

if [[ ! -d $CCROOT/system || ! -d $CCROOT/vhosts ]]; then
	echo "Copy failed, aborting"
	exit 1
fi

echo "Building setuid binary ..."
cd $CCROOT/system/runascc
rm -f runascc
rm -f castd

if [ $PHPNOARGS != "php" ]; then
	mv runascc.c runascc.c.default
	cat runascc.c.default | sed "s|define PHP_PATH .*|define PHP_PATH \"$PHPNOARGS\"|g" > runascc.c

	mv castd.c castd.c.default
	cat castd.c.default | sed "s|define PHP_PATH .*|define PHP_PATH \"$PHPNOARGS\"|g" > castd.c

fi

gcc -o runascc runascc.c
if [ ! -f runascc ]; then
	echo "Error occurred while attempting to compile setuid binary, aborting"
	exit 1
fi
chown ${INSTALLUSER}:${INSTALLGROUP} runascc
chmod 711 runascc
chmod u+s runascc

gcc -o castd castd.c
if [ ! -f castd ]; then
	echo "Error occurred while attempting to compile castd binary, aborting"
	exit 1
fi
[ $ISROOT -gt 0 ] && chown ${INSTALLUSER}:${INSTALLGROUP} castd
chmod 711 castd
chmod u+s castd

rm -f spawn
gcc -o spawn spawn.c
if [ ! -f spawn ]; then
	echo "Error occurred while attempting to compile spawn binary, aborting"
	exit 1
fi
chmod 711 spawn

echo "ident=$PHPNOARGS" > ./phpident
echo "cgi=$PHPCGI" >> ./phpident
echo -n "version=" >> ./phpident
$PHPNOARGS -v >> ./phpident

cd $CCROOT

echo "Copying files ..."
if [ ! -d $WEBROOT ]; then
	mkdir -p $WEBROOT
	CREATEDWEBROOT=1
fi

$COPYCMD $SELFPATH/www $WEBROOT/
mv $WEBROOT/www/* $WEBROOT/
rmdir $WEBROOT/www
mkdir -p $WEBROOT/ioncube
$COPYCMD $SELFPATH/ioncube $WEBROOT/

if [ $USESUPHP -gt 0 ]; then
	WEBROOTOWNER=`$STATCMD $WEBROOT`
else
	WEBROOTOWNER="${INSTALLUSER}:${INSTALLGROUP}"
fi

if [ $CREATEDWEBROOT -gt 0 ]; then
	chown -R $WEBROOTOWNER $WEBROOT
else
	chown -R $WEBROOTOWNER $WEBROOT/*
fi

TMPCWD=`pwd`
cd $CCROOT
ABSCCROOT=`pwd`
cd $TMPCWD

echo "<?php define('SYSTEM_PATH','$ABSCCROOT/system/'); ?>" > $WEBROOT/config.php

echo ""

echo "===== STREAMING SERVER SOFTWARE ==============================================="

SERVERTYPE=""
while [ -z "$SERVERTYPE" ]; do
	echo ""
	echo "Centova Cast can work with IceCast or ShoutCast streaming servers.  If you"
	echo "choose ShoutCast (recommended), this installer can download and install it"
	echo "for you automatically."
	echo ""
	echo "Select the type of server you want to use:"
	echo ""
	if [ ! -z "$CCSERVER" ]; then
		SERVERTYPE=$CCSERVER
		CCSERVER=""
		echo "Using: $SERVERTYPE"
	else
		echo "[S] Press S for ShoutCast (recommended)"
		echo "[I] Press I for IceCast"
		read -n 1 SERVERTYPE
	fi
	case "$SERVERTYPE" in
		[iI]|IceCast)
			SERVERTYPE=IceCast
			[ -z "$SERVERPATH" ] && SERVERPATH=`which icecast2 2>/dev/null`
			SERVERVAR=ICECAST_PATH
			;;
		[sS]|ShoutCast)	
			SERVERTYPE=ShoutCast
			[ -z "$SERVERPATH" ] && SERVERPATH=`which sc_serv 2>/dev/null`
			SERVERVAR=SHOUTCAST_PATH
			;;
		*) SERVERTYPE="";;
	esac
done
echo ""
echo ""

BINARYFOUND=0
if [ ! -z "$SERVERPATH" ]; then
	if [ -d $SERVERPATH ]; then
		# directory
		SERVERPATH=""
	elif [ -x $SERVERPATH ]; then
		BINARYFOUND=1
	fi
fi


if [ "$SERVERTYPE" == "ShoutCast" ]; then
	if [ ! -z "$HTTPGET" ]; then
		echo "===== SHOUTCAST DNAS LICENSE AGREEMENT ========================================"
		cat <<EOF

Before using the ShoutCast DNAS streaming server software, you must agree to
its license agreement at the following URL:
$SCLIC

EOF

		SHOUTLIC="$SHOUTAGREE"
		while [[ $SHOUTLIC != "y" && $SHOUTLIC != "Y" && $SHOUTLIC != "n" && $SHOUTLIC != "N" ]]; do
			cat <<EOF
Press Y if you accept this agreement and wish to install ShoutCast DNAS.
Press N to skip the installation of ShoutCast DNAS.

EOF
	
			echo "Do you accept these license terms? (y/n) "
			read -n 1 SHOUTLIC
			echo ""
			echo ""
		done
	
		if [[ $SHOUTLIC == "Y" || $SHOUTLIC == "y" ]]; then
			$HTTPGET sc_serv.tar.gz $SCSERV
			if [ -f sc_serv.tar.gz ]; then
				mkdir -p $SCPATH
				mv sc_serv.tar.gz $SCPATH
				pushd $SCPATH
				tar xzvf sc_serv.tar.gz
				rm -f sc_serv.tar.gz
				popd
				if [ -f ${SCPATH}sc_serv ]; then
					chown -R ${INSTALLUSER}:${INSTALLGROUP} $SCPATH
					chmod a+x ${SCPATH}sc_serv
					SERVERPATH="${SCPATH}sc_serv"
					BINARYFOUND=1
				else
					echo "Sorry, an error occurred while unpacking ShoutCast DNAS."
				fi
			else
				echo "Sorry, an error occurred while downloading ShoutCast DNAS."
			fi
			echo ""
		fi
	fi

	if [ -z "$SERVERPATH" ]; then
		echo "Centova Cast needs to know where your ShoutCast server software (sc_serv) has"
		echo "been installed so that it can launch it whenever necessary."
		echo ""
	fi
else
	echo "Centova Cast needs to know where your IceCast server application (usually named"
	echo "icecast or icecast2) has been installed so that it can launch it whenever"
	echo "necessary."
	echo ""
fi

[ $BINARYFOUND -gt 0 ] && echo "Using: $SERVERPATH"
while [ $BINARYFOUND -eq 0 ]; do
	echo -n "Enter the path and filename to your $SERVERTYPE binary [$SERVERPATH]: "
	read NEWSERVERPATH
	[ ! -z "$NEWSERVERPATH" ] && SERVERPATH="$NEWSERVERPATH"
	
	if [ -d $SERVERPATH ]; then
		BINARYFOUND=0
		echo "The path $SERVERPATH is a directory, not an executable file."

	elif [ -x $SERVERPATH ]; then
		BINARYFOUND=1

	else
		BINARYFOUND=0
		echo "The filename $SERVERPATH does not exist or is not an executable file."
	fi

done


SCRIPTABLE=0

SOURCETYPE=""
# autodetect ices-cc as that's the most likely choice
if [[ -z "$CCSOURCE" && -x /usr/local/ices/bin/ices ]]; then
	SOURCETYPE="ices"
	SOURCEVAR=ICES_PATH
	SOURCEPATH="/usr/local/ices/bin/ices"
fi

echo ""
echo ""
echo "===== STREAMING SOURCE SOFTWARE ==============================================="

while [ -z "$SOURCETYPE" ]; do
	echo ""
	echo "If you plan to use Centova Cast's 'AutoDJ' features, you must install a"
	echo "streaming source application."
	
	if [[ ! -z "$HTTPGET" && ! -z "$DIST" ]]; then
		echo ""
		echo "If you choose ices-cc (recommended), this installer can download and install it"
		echo "for you automatically."
		echo ""
		AUTOICES="$DOWNLOADICES"
		while [[ $AUTOICES != "y" && $AUTOICES != "Y" && $AUTOICES != "n" && $AUTOICES != "N" ]]; do
			echo -n "Do you want to download and install ices-cc automatically now? (y/n) "
			read -n 1 AUTOICES
			echo ""
			echo ""
		done
	
		if [[ $AUTOICES == "Y" || $AUTOICES == "y" ]]; then
			UNSUPPORTED=0
			[[ $HASREDHAT -eq 0 && $HASDEBIAN -eq 0 ]] && UNSUPPORTED=1
			[ $IS64BIT -gt 0 ] && UNSUPPORTED=1

			if [[ $UNSUPPORTED -gt 0 && $CANBUILDICES -eq 0 ]]; then
				echo ""
				echo "Prebuilt ices-cc packages are not available for the Linux distribution or architecture"
				echo "you are using, but the Centova Cast installer can optionally attempt to build a custom"
				echo "package for you."
				echo ""
				echo "Note that this requires that you have installed a proper GNU build toolchain (gcc, make,"
				echo "etc.) on your server."
				echo ""

				DOBUILD="x"
				while [[ $DOBUILD != "y" && $DOBUILD != "Y" && $DOBUILD != "n" && $DOBUILD != "N" ]]; do
					echo -n "Should the installer attempt to compile a custom ices-cc build for you? (y/n)"
					read -n 1 DOBUILD
					echo ""
					echo ""
				done			
				if [[ $DOBUILD == "Y" || $DOBUILD == "y" ]]; then
					CANBUILDICES=1
				fi
			
			fi


			if [ $HASREDHAT -gt 0 ]; then
				$HTTPGET ices.rpm http://www.centova.com/clientdist/ices/latest/ices-cc-latest.${DIST}.i386.rpm
				rpm -Uvh ices.rpm
				[ $? -gt 0 ] && echo "An error occurred while attempting to install ices-cc from RPM; aborting" && exit 1
				rm -f ices.rpm
				
			elif [ $HASDEBIAN -gt 0 ]; then
				$HTTPGET ices.deb http://www.centova.com/clientdist/ices/latest/ices-cc_latest.i386.deb
				dpkg -i ices.deb
				[ $? -gt 0 ] && echo "An error occurred while attempting to install ices-cc from DEB; aborting" && exit 1
				rm -f ices.deb
				
			elif [ $CANBUILDICES -gt 0 ]; then
			
				$HTTPGET buildices.sh http://www.centova.com/clientdist/ices/buildices.sh
				chmod +x ./buildices.sh
				./buildices.sh --proceed --buildlame=1 --srcpath=/usr/local/src/ices/ --prefix=/usr/local/ices/
				
			else
				echo "Error: Unsupported (or 64-bit) Linux distribution; aborting."
				echo ""
				echo "You may want to use --canbuildices to build ices from source"
				echo "if you have a build toolchain installed."

				exit 1
			fi

			if [ -x /usr/local/ices/bin/ices ]; then
				CCSOURCE="ices"
				SOURCEVAR=ICES_PATH
				SOURCEPATH="/usr/local/ices/bin/ices"
			fi
			echo ""
		fi
	fi
	
	echo ""
	if [ ! -z "$CCSOURCE" ]; then
		SOURCETYPE="$CCSOURCE"
		CCSOURCE=""
		echo "Using: $SOURCETYPE"
	else
		echo "Select the type of source you have installed for use with Centova Cast:"
		echo ""
	
		if [ "x$SERVERTYPE" == "xIceCast" ]; then
			echo "[I] Press I for ices-cc v0.4.1 (strongly recommended) or ices v0.4"
			echo "[E] Press E for ezstream v0.4+"
			echo "[C] Press C for ices v2.0 (streams in Ogg format only - NO MP3 SUPPORT)"
			echo "[N] Press N if you will not be using AutoDJ features"
		else
			echo "[I] Press I for ices-cc v0.4.1 (strongly recommended) or ices v0.4"
			echo "[S] Press S for sc_trans"
			echo "[N] Press N if you will not be using AutoDJ features"
		fi
		read -n 1 SOURCETYPE
	fi
	case "$SOURCETYPE" in
		[iI]|ices)	SOURCETYPE=ices
			[ -z "$SOURCEPATH" ] && SOURCEPATH=`which ices 2>/dev/null`
			[ -z "$SOURCEPATH" ] && SOURCEPATH=`ls /usr/local/ices/bin/ices 2>/dev/null`
			SOURCEVAR=ICES_PATH
			;;
		[cC]|ices2)	
			if [ "x$SERVERTYPE" != "xShoutCast" ]; then
				SOURCETYPE=ices2
				[ -z "$SOURCEPATH" ] && SOURCEPATH=`which ices2 2>/dev/null`
				[ -z "$SOURCEPATH" ] && SOURCEPATH=`ls /usr/local/ices2/bin/ices2 2>/dev/null`
				SOURCEVAR=ICES2_PATH
			else
				SOURCETYPE=""
			fi
			;;
		[eE]|ezstream)	
			if [ "x$SERVERTYPE" != "xShoutCast" ]; then
				SOURCETYPE=ezstream
				[ -z "$SOURCEPATH" ] && SOURCEPATH=`which ezstream 2>/dev/null`
				SOURCEVAR=EZSTREAM_PATH
			else
				SOURCETYPE=""
			fi
			;;			
		[sS]|sctrans)
			if [ "x$SERVERTYPE" != "xIceCast" ]; then
				SOURCETYPE=sctrans
				[ -z "$SOURCEPATH" ] && SOURCEPATH=`which sc_trans 2>/dev/null`
				SOURCEVAR=SCTRANS_PATH
			else
				SOURCETYPE=""
			fi
			;;
		[nN]|none)	SOURCETYPE=none
			SOURCEPATH=""
			SOURCEVAR="SOURCE_PATH"
			;;
		*) SOURCETYPE="";;
	esac
	echo ""
done


if [ "x$SOURCETYPE" != "xnone" ]; then
	BINARYFOUND=0

	if [ ! -z "$SOURCEPATH" ]; then
		if [ -d $SOURCEPATH ]; then
			# directory
			SOURCEPATH=""
		elif [ -x $SOURCEPATH ]; then
			BINARYFOUND=1
		fi
	fi
	
	echo ""
	echo "Centova Cast needs to know where your $SOURCETYPE source application has been"
	echo "installed so that it can launch it whenever necessary."
	echo ""

	[ $BINARYFOUND -gt 0 ] && echo "Using: $SOURCEPATH"
	while [ $BINARYFOUND -eq 0 ]; do
		echo -n "Enter the path and filename to your $SOURCETYPE binary [$SOURCEPATH]: "
		read NEWSOURCEPATH
		[ ! -z "$NEWSOURCEPATH" ] && SOURCEPATH="$NEWSOURCEPATH"
		
		if [ -d $SOURCEPATH ]; then
			BINARYFOUND=0
			echo "The path $SOURCEPATH is a directory, not an executable file."

		elif [ -x $SOURCEPATH ]; then
			BINARYFOUND=1

		else
			BINARYFOUND=0
			echo "The filename $SOURCEPATH does not exist or is not an executable file."
		fi
	
	done

	if [ "x$SOURCETYPE" == "xices" ]; then
		ISICESCC=`$SOURCEPATH -V | grep '^ices-cc'`
		[ ! -z "$ISICESCC" ] && SOURCETYPE="icescc"
	fi

fi

if [ "$USESUPHP" -gt 0 ]; then
	ENABLEDAEMON="true"
	ENABLERUNASCC="false"
else
	ENABLERUNASCC="true"
	ENABLEDAEMON="false"
fi

DBNAME=""
DBUSER=""
DBPASS=""
DBHOST=""

INSTALLUID=`id -u ${INSTALLUSER}`;
INSTALLGID=`id -g ${INSTALLUSER}`;

echo "<?php
define('NOT_INSTALLED','${INSTALLUID}|${INSTALLGID}');

// Set this to true if using the setuid 'runascc' binary
// (and associated 'spawn' binary), otherwise set it to false.
define('USE_RUNASCC',${ENABLERUNASCC});

// Use these settings to use Centova Cast's daemon instead of
// attempting to launch processes directly from the web server.
// This is necessary for suPHP servers.
define('USE_DAEMON',${ENABLEDAEMON});
define('DAEMON_HOST','127.0.0.1');
define('DAEMON_PORT',2199);

// Enter your database connection information.
define('DB_NAME','${DBNAME}');
define('DB_USER','${DBUSER}');
define('DB_PASS','${DBPASS}');
define('DB_HOST','${DBHOST}');

// Enter the path to your CentovaCast installation root.  (This
// directory should contain your system/ and vhost/ directories.)
define('CAST_PATH','${ABSCCROOT}/');

// Due to licensing restrictions we cannot distribute the free
// MiniCaster web-based audio player with Centova Cast.  However,
// you can download it from:
// http://www.draftlight.net/dnex/mp3player/minicaster/free/
//
// Then just upload the minicaster.swf file to your Centova Cast web
// root, uncomment the following line, and set it to the URI to
// minicaster.swf, eg:
//  define('MINICASTER_SWF','/minicaster.swf');
//
// This will instantly activate the web-based player on all of your
// clients' start pages and allow your clients' listeners to tune
// in using their web browsers.
define('MINICASTER_SWF','');

// Set the locale to use for Centova Cast's internationalization
// features.  Supported locales are stored under system/locale/.
define('LOCALE','en_US');


// ==== Advanced settings ================================================
// Enter the metadata format for use with ices-cc realtime track
// selection.  This will display in the listener's media player,
// the recent tracks list, and so-on.
// define('ICES_METADATA_FORMAT','%artist% - %album% (%releaseyear%) - %title% (%bitrate%kbps, %length% seconds)');

// Set the frequency (in hours) at which the logs for each account will
// be processed (to update the stats in the client control panels).
define('LOGPROCESS_FREQUENCY',12);

// Set the frequency (in hours) at which the logs for each account will
// be rotated.  For best performanc this should be identical to
// LOGPROCESS_FREQUENCY.
define('LOGROTATE_FREQUENCY',12);

// Sets the maximum number of archived log files to keep (i.e.,
// error.log.1, error.log.2, etc.) after the logs are rotated.
define('MAX_ARCHIVED_LOGS',4);

// if enabled, this Centova Cast installation will proxy API requests
// to the single sign-on servers configured in the admin area to
// balance the provisioning load
define('ENABLE_API_CLUSTERING',false);

// if enabled, the internal on-demand-media proxy will always be used
// instead of using ShoutCast's internal web server
define('FORCE_ONDEMAND_PROXY',false);

// if enabled, clients will be permitted to edit their own encoder settings
// (including sample rate, channels, and so-on)
define('CLIENT_ENCODER_SETTINGS',false);

// if enabled, clients will be permitted to edit their own bit rate limit
define('CLIENT_BITRATE_MODIFY',false);

// if enabled, the event scripts under /home/centovacast/system/scripts will
// be called in response to various events within Centova Cast
define('EVENT_SCRIPTS',false);

// if enabled, Centova Cast will follow symlinks when calculating disk space
// usage; otherwise, symlinks will be ignored
define('DISK_USAGE_SYMLINKS',true);

// if enabled, Centova Cast will follow symlinks when importing media into
// the user's media library; otherwise, symlinks will be ignored
define('INDEX_SYMLINKS',true);

// Set this to TRUE if self-referencing links (such as the link to the
// ShoutCast administration page, and the links to tune into the server)
// should use the hostname configured for the stream instead of its IP address.
// Note that enabling this WILL BREAK YOUR STREAMS if you have not correctly
// configured DNS for the hostnames you've specified.  DO NOT ENABLE THIS
// UNLESS YOU FULLY UNDERSTAND THE CONSEQUENCES OF DOING SO.
define('SELFREF_HOSTNAME',false);

// Set this to an IP address or hostname to override the hostname portion
// of all self-referencing links.  Note that before enabling this, you should
// ensure that:
// 1) All of your streams are configured to listen on the same IP address, and
// 2) The IP address or hostname below corresponds to the IP address on which
//    your streams are configured to listen.
// This may be useful for NAT configurations or to force a specific hostname
// for other purposes.
//
// Example: define('SELFREF_OVERRIDE','foo.example.com');
define('SELFREF_OVERRIDE','');

// By default, the disk usage graph at the top of the accounts page in the admin
// area is based on the free space on whichever partition holds your Centova Cast
// "vhosts" directory.
//
// If you need to check the space on another partition instead (e.g. if you use
// symlinks to reference media on another partition), specify the path to the
// alternate partition here.
define('MEDIA_PARTITION','');

// Specifies the maximum width and height for cover images shown on stream start
// pages.  Images will be resized to these dimensions.
define('COVER_MAX_WIDTH',50);
define('COVER_MAX_HEIGHT',43);

// Enables experimental portability mode; refer to:
// http://www.centova.com/pages/faqs/display/centova_cast_on_freebsd_55
define('PORTABLE',false);

// Limits identical song playbacks within a given period; refer to:
// https://secure.centova.com/pages/faqs/display/limiting_performances_of_a_95
define('RANDOM_UNIQUE_PERIOD',360); // minutes

// If enabled, users will be permitted to upload M3U playlists.  This option is
// disabled by default because it is NOT SUPPORTED IN ANY CAPACITY by Centova 
// Technologies.  Read this article before enabling:
// http://www.centova.com/pages/faqs/display/can_i_create_an_m3u_playlist_96
define('ALLOW_IMPORT_M3U',false);

// Controls whether Centova Cast will periodically retrieve news from
// centova.com to display in the admin area.
define('FETCH_NEWS',true);

// Controls how often Centova Cast will poll each streaming server for its
// state and current song title.  For performance reasons, values under 60
// seconds are not recommended.
define('SERVSTATE_CACHE_TIMEOUT',60);

// If enabled, the user's disk quota will only apply to files uploaded to the
// /home/centovacast/vhosts/USERNAME/var/spool/ directory and its subdirectories;
// any configuration files, log files, etc. will be excluded from the quota
define('QUOTA_SPOOL_ONLY',false);
?>" > $CCROOT/system/config.php

echo "
default_server_type=${SERVERTYPE}
default_server_path=${SERVERPATH}
default_source_type=${SOURCETYPE}
default_source_path=${SOURCEPATH}
" > $CCROOT/system/webinstaller.dat

[ $ISROOT -gt 0 ] && chown $INSTALLUSER:$INSTALLGROUP $CCROOT/system/config.php

echo ""

echo "Checking direct server communication ..."
CHECKRESULT=`$PHPBINARY -q $CCROOT/system/ccmanage.php version check | grep '^OK'`
if [ -z "$CHECKRESULT" ]; then
	echo "Server communication check failed.  This likely means that the installation"
	echo "was not successful."
	echo ""
	echo "The error output was:"
	$PHPBINARY -q $CCROOT/system/ccmanage.php version check
	echo ""
	echo "Installation failed."
	exit 1
fi

if [ "$USESUPHP" -gt 0 ]; then
	$CCROOT/system/runascc/spawn $CCROOT/system/runascc/castd > $CCROOT/vhosts/castd.pid
	chown ${INSTALLUSER}:${INSTALLGROUP} $CCROOT/vhosts/castd.pid
else

	echo "Checking setuid binary server communication ..."
	CHECKRESULT=`$CCROOT/system/runascc/runascc exec ccmanage version check | grep '^OK'`
	if [ -z "$CHECKRESULT" ]; then
		echo "Setuid binary server communication check failed.  This indicates that the"
		echo "Centova Cast installation completed successfully, but there is a problem"
		echo "with the setuid binary that needs to be resolved before Centova Cast can"
		echo "be used."
		echo ""
		echo "The error output was:"
		$CCROOT/system/runascc/runascc exec ccmanage version check
		echo ""
		echo "Installation failed."
		exit 1
	fi

fi

CRONJOBEXISTS=`cat /etc/crontab | grep -v '^#' | grep -c 'ccmanage cronjob'`
[ $CRONJOBEXISTS -eq 0 ] && echo "*/5 * * * * $INSTALLUSER $CCROOT/system/runascc/runascc exec ccmanage cronjob all >/dev/null" >> /etc/crontab


FTPPURE=0
FTPPRO=0
# pure-ftpd can be setup in so many ways that we can only do it when it's a
# cpanel server, as cpanel is reasonably consistent in how it configures it
[[ $HASCPANEL -gt 0 && -x /etc/init.d/pure-ftpd ]] && FTPPURE=1
# da uses authuserfile for itself, so that kinda screws us
[[ -f /etc/proftpd.conf || -f /etc/proftpd/proftpd.conf ]] && FTPPRO=1
[ $HASDA -gt 0 ] && FTPPRO=0

# if we've got pure or pro, prompt for autoinstall
if [[ $FTPPURE -gt 0 || $FTPPRO -gt 0 ]]; then
	echo ""
	echo ""
	echo "===== FTP SERVER INTEGRATION =================================================="
	echo ""
	echo "This installer can automatically configure Centova Cast to work with your"
	echo "existing FTP server.  Once this is done, your users will be able to login to"
	echo "your FTP server using their Centova Cast username and password."
	echo ""
	
	while [[ $FTPAUTO != "y" && $FTPAUTO != "Y" && $FTPAUTO != "n" && $FTPAUTO != "N" ]]; do
		echo -n "Setup FTP integration automatically now? (y/n) "
		read -n 1 FTPAUTO
		echo ""
		echo ""
	done
	
	
	if [[ $FTPAUTO == "Y" || $FTPAUTO == "y" ]]; then

		# setup Pure-FTPd
		if [ $FTPPURE -gt 0 ]; then
			
			echo "Configuring Pure-FTPd ..."
		
			cat $SELFPATH/ftp/pure-ftpd/non-mysql/centovacast-pureauth \
		        | sed "s|define('CENTOVACAST_PATH',.*|define('CENTOVACAST_PATH','${CCROOT}/system/');|g" \
				> /usr/sbin/centovacast-pureauth
			chmod +x /usr/sbin/centovacast-pureauth
			
			cp -f /etc/init.d/pure-ftpd /etc/init.d/pure-ftpd.pre-centovacast
			cat /etc/init.d/pure-ftpd.pre-centovacast | sed 's|/usr/sbin/pureauth|/usr/sbin/centovacast-pureauth|g' > /etc/init.d/pure-ftpd

			/etc/init.d/pure-ftpd restart >/dev/null 2>&1
		fi
		
		# setup ProFTPd
		if [ $FTPPRO -gt 0 ]; then
		
			echo "Configuring ProFTPd ..."
		
			PROCFG=/etc/proftpd.conf
			[[ ! -f $PROCFG && -f /etc/proftpd/proftpd.conf ]] && PROCFG=/etc/proftpd/proftpd.conf
		
			HASAUTHORDER=`cat $PROCFG | grep -c '^AuthOrder'`
			HASAUTHUSER=`cat $PROCFG | grep -c '^AuthUserFile'`
			HASAUTHGROUP=`cat $PROCFG | grep -c '^AuthGroupFile'`
			
			
			AUTHUSERISCC=`cat $PROCFG | grep '^AuthUserFile' | grep -c 'centovacast'`
			
			if [ $AUTHUSERISCC -gt 0 ]; then
				cat <<EOF

ERROR: Your ProFTPd installation is already setup to authenticate against
another user list.  It appears to be a Centova Cast user list.

If you have previously installed Centova Cast in the same location on this
server, then this is harmless -- you can ignore this message and FTP 
integration should work fine.  Otherwise, you may need to manually set up
FTP integration.
EOF
				if [[ $AUTOINSTALL -eq 0 || $CPINSTALL -gt 0 ]]; then
					echo ""
					echo "Press ENTER to continue"
					read
				fi
			
			
			elif [ $HASAUTHUSER -gt 0 ]; then
				cat <<EOF

ERROR: Your ProFTPd installation is already setup to authenticate against
another user list.  If you have previously installed Centova Cast on this
server, this is harmless -- you can ignore this message and FTP integration
will work fine.

If you are using other software on this server (such as the DirectAdmin
web hosting control panel) which uses ProFTPd's AuthUserFile feature,
proceeding would break the existing integration mechanism, so Centova
Cast FTP integration has been aborted.
EOF
				if [[ $AUTOINSTALL -eq 0 || $CPINSTALL -gt 0 ]]; then
					echo ""
					echo "Press ENTER to continue"
					read
				fi
			else
			
				mkdir -p $CCROOT/ftp
				cat $SELFPATH/ftp/proftpd/non-mysql/centovacast-userdump.php \
			        | sed "s|define('CENTOVACAST_PATH',.*|define('CENTOVACAST_PATH','${CCROOT}/system/');|g" \
					> $CCROOT/ftp/centovacast-userdump.php
				chown -R ${INSTALLUSER}:${INSTALLGROUP} $CCROOT/ftp
				
				CRONJOBEXISTS=`cat /etc/crontab | grep -v '^#' | grep -c 'centovacast-userdump'`
				[ $CRONJOBEXISTS -eq 0 ] &&	echo "*/5 * * * * $INSTALLUSER $PHPBINARY -q $CCROOT/ftp/centovacast-userdump.php /home/centovacast/ftp/passwd /home/centovacast/ftp/group" >> /etc/crontab
				
				cp -f $PROCFG ${PROCFG}.pre-centovacast
				
				if [ $HASAUTHORDER -gt 0 ]; then
					HASAUTHFILE=`cat $PROCFG | grep '^AuthOrder' | grep -c 'mod_auth_file.c'`
					if [ $HASAUTHFILE -eq 0 ]; then
						cat $PROCFG \
							| sed 's/^AuthOrder \(.*\)/AuthOrder \1 mod_auth_file.c/g' \
							> ${PROCFG}.tmp
						mv -f ${PROCFG}.tmp $PROCFG
					fi
				else
					echo "AuthOrder mod_auth_unix.c mod_auth_file.c" >> $PROCFG
				fi
				echo "AuthUserFile /home/centovacast/ftp/passwd" >> $PROCFG
				[ $HASAUTHGROUP -eq 0 ] && echo "AuthGroupFile /home/centovacast/ftp/group" >> $PROCFG
				echo "DefaultRoot ~" >> $PROCFG
	
				/etc/init.d/proftpd restart >/dev/null 2>&1
			fi
		fi
		
	fi
	
	
fi


echo ""
echo ""
echo "===== COMPLETE ================================================================"
echo ""
echo "Control panel files installed."
echo ""
echo "You must now open the Centova Cast web-based control panel in your web browser"
echo "to finish the installation and configure Centova Cast."
echo ""
echo "Thank you for using Centova Cast."

exit 0